@@ -7,10 +7,10 @@ use std::fmt::{Debug, Display, Error, Formatter};
 #[derive(Debug, Clone, Hash, Eq, PartialEq, Ord, PartialOrd, Serialize, Deserialize)]
 pub enum Terminal {
     ///
-    /// A physical terminal symbol
+    /// A physical terminal symbol with the scanner states it belongs to
     /// Entities that are provided by the lexer.
     ///
-    Trm(String),
+    Trm(String, Vec<usize>),
 
     ///
     /// Epsilon symbol, the empty word
@@ -27,11 +27,11 @@ pub enum Terminal {
 }
 
 impl Terminal {
-    pub fn t(t: &str) -> Self {
-        Self::Trm(t.to_owned())
+    pub fn t(t: &str, s: Vec<usize>) -> Self {
+        Self::Trm(t.to_owned(), s)
     }
     pub fn is_trm(&self) -> bool {
-        matches!(self, Self::Trm(_))
+        matches!(self, Self::Trm(_, _))
     }
     pub fn is_eps(&self) -> bool {
         matches!(self, Self::Eps)
@@ -42,17 +42,50 @@ impl Terminal {
 
     pub fn create(s: &Symbol) -> Self {
         match s {
-            Symbol::T(Terminal::Trm(t)) => Terminal::Trm(t.to_string()),
+            Symbol::T(Terminal::Trm(t, s)) => Terminal::Trm(t.to_string(), s.to_vec()),
             Symbol::T(Terminal::End) => Terminal::End,
             _ => panic!("Unexpected symbol type: {:?}", s),
         }
     }
+
+    pub fn add_scanner(&mut self, sc: usize) {
+        match self {
+            Terminal::Trm(_, s) => {
+                if !s.contains(&sc) {
+                    s.push(sc);
+                    s.sort_unstable();
+                }
+            }
+            _ => panic!("Unexpected symbol type: {:?}", self),
+        }
+    }
+
+    ///
+    /// Get the scanner state in front of the terminal
+    ///
+    pub fn format<R>(&self, scanner_state_resolver: R) -> String
+    where
+        R: Fn(&[usize]) -> String,
+    {
+        match self {
+            Self::Trm(t, s) => {
+                if *s == vec![0] {
+                    // Don't print state if terminal is only in state INITIAL (0)
+                    format!("\"{}\"", t)
+                } else {
+                    format!("<{}>\"{}\"", scanner_state_resolver(s), t)
+                }
+            }
+            Self::Eps => "\u{03B5}".to_string(), // Lower creek letter Epsilon (ε)
+            Self::End => "$".to_string(),
+        }
+    }
 }
 
 impl Display for Terminal {
     fn fmt(&self, f: &mut Formatter<'_>) -> Result<(), Error> {
         match self {
-            Self::Trm(t) => write!(f, "\"{}\"", t),
+            Self::Trm(t, _) => write!(f, "\"{}\"", t),
             Self::Eps => write!(f, "\u{03B5}"), // Lower creek letter Epsilon (ε)
             Self::End => write!(f, "$"),
         }
@@ -86,12 +119,18 @@ pub enum Symbol {
 
     ///
     /// Terminal symbol of the grammar.
+    ///
     T(Terminal),
+
+    ///
+    /// Instruction to switch scanner state
+    ///
+    S(usize),
 }
 
 impl Symbol {
-    pub fn t(t: &str) -> Self {
-        Self::T(Terminal::Trm(t.to_owned()))
+    pub fn t(t: &str, s: Vec<usize>) -> Self {
+        Self::T(Terminal::Trm(t.to_owned(), s))
     }
     pub fn n(n: &str) -> Self {
         Self::N(n.to_owned())
@@ -99,6 +138,9 @@ impl Symbol {
     pub fn e() -> Self {
         Self::T(Terminal::End)
     }
+    pub fn s(s: usize) -> Self {
+        Self::S(s)
+    }
     pub fn is_t(&self) -> bool {
         matches!(self, Self::T(_))
     }
@@ -108,6 +150,9 @@ impl Symbol {
     pub fn is_end(&self) -> bool {
         matches!(self, Self::T(Terminal::End))
     }
+    pub fn is_switch(&self) -> bool {
+        matches!(self, Self::S(_))
+    }
     pub fn get_t(&self) -> Option<Terminal> {
         if let Self::T(t) = &self {
             Some(t.clone())
@@ -136,6 +181,17 @@ impl Symbol {
             None
         }
     }
+
+    pub fn format<R>(&self, scanner_state_resolver: &R) -> String
+    where
+        R: Fn(&[usize]) -> String,
+    {
+        match self {
+            Self::N(n) => n.to_string(),
+            Self::T(t) => t.format(scanner_state_resolver),
+            Self::S(s) => scanner_state_resolver(&[*s]),
+        }
+    }
 }
 
 impl Display for Symbol {
@@ -143,6 +199,7 @@ impl Display for Symbol {
         match self {
             Self::N(n) => write!(f, "{}", n),
             Self::T(t) => write!(f, "{}", t),
+            Self::S(s) => write!(f, "S({})", s),
         }
     }
 }
