@@ -1,7 +1,7 @@
 use crate::parser::parol_grammar_trait::ParolGrammarTrait;
 use id_tree::Tree;
 use log::trace;
-use parol_runtime::parser::errors::*;
+use parol_runtime::errors::*;
 use parol_runtime::parser::{ParseTreeStackEntry, ParseTreeType};
 use std::fmt::{Debug, Display, Error, Formatter};
 
@@ -15,8 +15,9 @@ pub enum Factor {
     Group(Alternations),
     Repeat(Alternations),
     Optional(Alternations),
-    Terminal(String),
+    Terminal(String, Vec<usize>),
     NonTerminal(String),
+    ScannerSwitch(usize),
 }
 
 impl Display for Factor {
@@ -25,8 +26,17 @@ impl Display for Factor {
             Self::Group(g) => write!(f, "({})", g),
             Self::Repeat(r) => write!(f, "{{{}}}", r),
             Self::Optional(o) => write!(f, "[{}]", o),
-            Self::Terminal(t) => write!(f, "T({})", t),
+            Self::Terminal(t, s) => write!(
+                f,
+                "<{}>T({})",
+                s.iter()
+                    .map(|s| format!("{}", s))
+                    .collect::<Vec<String>>()
+                    .join(", "),
+                t
+            ),
             Self::NonTerminal(n) => write!(f, "N({})", n),
+            Self::ScannerSwitch(n) => write!(f, "S({})", n),
         }
     }
 }
@@ -111,12 +121,22 @@ impl Display for Production {
     }
 }
 
+#[derive(Debug, Clone)]
+pub struct ScannerConfig {
+    pub name: String,
+    pub line_comments: Vec<String>,
+    pub block_comments: Vec<(String, String)>,
+    pub auto_newline_off: bool,
+    pub auto_ws_off: bool,
+}
+
 #[derive(Debug, Clone)]
 pub enum ParolGrammarItem {
     Prod(Production),
     Alts(Alternations),
     Alt(Alternation),
     Fac(Factor),
+    StateList(Vec<usize>),
 }
 
 impl Display for ParolGrammarItem {
@@ -126,6 +146,36 @@ impl Display for ParolGrammarItem {
             Self::Alts(a) => write!(f, "{}", a),
             Self::Alt(a) => write!(f, "{}", a),
             Self::Fac(t) => write!(f, "{}", t),
+            Self::StateList(s) => write!(
+                f,
+                "SL<{}>",
+                s.iter()
+                    .map(|e| format!("{}", e))
+                    .collect::<Vec<String>>()
+                    .join(", ")
+            ),
+        }
+    }
+}
+
+impl Display for ScannerConfig {
+    fn fmt(&self, f: &mut Formatter<'_>) -> std::result::Result<(), Error> {
+        write!(f, "name: {};", self.name)?;
+        write!(f, "line_comments: {:?};", self.line_comments)?;
+        write!(f, "block_comments: {:?};", self.block_comments)?;
+        write!(f, "auto_newline_off: {};", self.auto_newline_off)?;
+        write!(f, "auto_ws_off: {};", self.auto_ws_off)
+    }
+}
+
+impl Default for ScannerConfig {
+    fn default() -> Self {
+        Self {
+            name: "INITIAL".to_owned(),
+            line_comments: Vec::new(),
+            block_comments: Vec::new(),
+            auto_newline_off: false,
+            auto_ws_off: false,
         }
     }
 }
@@ -133,16 +183,27 @@ impl Display for ParolGrammarItem {
 ///
 /// Data structure used to build up a parol::GrammarConfig during parsing.
 ///
-#[derive(Debug, Clone, Default)]
+#[derive(Debug, Clone)]
 pub struct ParolGrammar {
-    pub ast_stack: Vec<ParolGrammarItem>,
+    pub item_stack: Vec<ParolGrammarItem>,
     pub title: Option<String>,
     pub comment: Option<String>,
     pub start_symbol: String,
-    pub line_comments: Vec<String>,
-    pub block_comments: Vec<(String, String)>,
-    pub auto_newline_off: bool,
-    pub auto_ws_off: bool,
+    pub scanner_configurations: Vec<ScannerConfig>,
+    current_scanner: ScannerConfig,
+}
+
+impl Default for ParolGrammar {
+    fn default() -> Self {
+        Self {
+            item_stack: Vec::new(),
+            title: None,
+            comment: None,
+            start_symbol: String::default(),
+            scanner_configurations: vec![ScannerConfig::default()],
+            current_scanner: ScannerConfig::default(),
+        }
+    }
 }
 
 impl ParolGrammar {
@@ -154,12 +215,12 @@ impl ParolGrammar {
     // Use this function for debugging purposes:
     // $env:RUST_LOG="parol::parser=trace"
     //
-    // trace!("{}", self.trace_ast_stack(context));
-    fn trace_ast_stack(&self, context: &str) -> String {
+    // trace!("{}", self.trace_item_stack(context));
+    fn trace_item_stack(&self, context: &str) -> String {
         format!(
-            "Ast stack at {}:\n{}",
+            "Item stack at {}:\n{}",
             context,
-            self.ast_stack
+            self.item_stack
                 .iter()
                 .rev()
                 .map(|s| format!("  {}", s))
@@ -167,6 +228,54 @@ impl ParolGrammar {
                 .join("\n")
         )
     }
+
+    fn handle_scanner_state(&mut self, context: &str) -> Result<()> {
+        let l = self.pop(context);
+        let s = self.pop(context);
+
+        match (&l, &s) {
+            (
+                Some(ParolGrammarItem::StateList(l)),
+                Some(ParolGrammarItem::Fac(Factor::NonTerminal(s))),
+            ) => {
+                if let Some(scanner_state) = self
+                    .scanner_configurations
+                    .iter()
+                    .position(|sc| sc.name == *s)
+                {
+                    let mut l = l.clone();
+                    l.push(scanner_state);
+                    self.push(ParolGrammarItem::StateList(l), context);
+                    trace!("{}", self.trace_item_stack(context));
+                    Ok(())
+                } else {
+                    Err(format!("{}: Unknown scanner name '{}'", context, s).into())
+                }
+            }
+            _ => Err(format!(
+                "{}: Expected [StateList, Factor::NonTerminal] on TOS, found [{:?}, {:?}]",
+                context, l, s
+            )
+            .into()),
+        }
+    }
+
+    fn push(&mut self, item: ParolGrammarItem, context: &str) {
+        trace!("push   {}: {}", context, item);
+        self.item_stack.push(item)
+    }
+
+    fn pop(&mut self, context: &str) -> Option<ParolGrammarItem> {
+        if !self.item_stack.is_empty() {
+            let item = self.item_stack.pop();
+            if let Some(ref item) = item {
+                trace!("pop    {}: {}", context, item);
+            }
+            item
+        } else {
+            None
+        }
+    }
 }
 
 impl Display for ParolGrammar {
@@ -174,14 +283,20 @@ impl Display for ParolGrammar {
         writeln!(f, "title: {:?}", self.title)?;
         writeln!(f, "comment: {:?}", self.comment)?;
         writeln!(f, "start_symbol: {}", self.start_symbol)?;
-        writeln!(f, "line_comments: {:?}", self.line_comments)?;
-        writeln!(f, "block_comments: {:?}", self.block_comments)?;
-        writeln!(f, "auto_newline_off: {:?}", self.auto_newline_off)?;
-        writeln!(f, "auto_ws_off: {:?}", self.auto_ws_off)?;
+        writeln!(f, "current_scanner: {}", self.current_scanner)?;
         writeln!(
             f,
             "{}",
-            self.ast_stack
+            self.scanner_configurations
+                .iter()
+                .map(|s| format!("{}", s))
+                .collect::<Vec<String>>()
+                .join("\n")
+        )?;
+        writeln!(
+            f,
+            "{}",
+            self.item_stack
                 .iter()
                 .map(|e| format!("{}", e))
                 .collect::<Vec<String>>()
@@ -191,18 +306,18 @@ impl Display for ParolGrammar {
 }
 
 impl ParolGrammarTrait for ParolGrammar {
-    /// Semantic action for production 7:
+    /// Semantic action for production 2:
     ///
     /// StartDeclaration: "%start" Identifier;
     ///
-    fn start_declaration_7(
+    fn start_declaration_2(
         &mut self,
-        _percent_start_0: &ParseTreeStackEntry,
+        _end_of_input_0: &ParseTreeStackEntry,
         _identifier_1: &ParseTreeStackEntry,
         _parse_tree: &Tree<ParseTreeType>,
     ) -> Result<()> {
-        let context = "start_declaration_7";
-        if let Some(ParolGrammarItem::Fac(Factor::NonTerminal(s))) = self.ast_stack.pop() {
+        let context = "start_declaration_2";
+        if let Some(ParolGrammarItem::Fac(Factor::NonTerminal(s))) = self.pop(context) {
             self.start_symbol = s;
             Ok(())
         } else {
@@ -210,18 +325,30 @@ impl ParolGrammarTrait for ParolGrammar {
         }
     }
 
-    /// Semantic action for production 8:
+    /// Semantic action for production 4:
+    ///
+    /// Declarations: ;
+    ///
+    fn declarations_4(&mut self, _parse_tree: &Tree<ParseTreeType>) -> Result<()> {
+        let context = "declarations_4";
+        trace!("{}", self.trace_item_stack(context));
+        self.scanner_configurations[0] = self.current_scanner.clone();
+        self.current_scanner = ScannerConfig::default();
+        Ok(())
+    }
+
+    /// Semantic action for production 5:
     ///
     /// Declaration: "%title" String;
     ///
-    fn declaration_8(
+    fn declaration_5(
         &mut self,
         _percent_title_0: &ParseTreeStackEntry,
         _string_1: &ParseTreeStackEntry,
         _parse_tree: &Tree<ParseTreeType>,
     ) -> Result<()> {
-        let context = "declaration_8";
-        if let Some(ParolGrammarItem::Fac(Factor::Terminal(s))) = self.ast_stack.pop() {
+        let context = "declaration_5";
+        if let Some(ParolGrammarItem::Fac(Factor::Terminal(s, _))) = self.pop(context) {
             self.title = Some(s);
             Ok(())
         } else {
@@ -229,59 +356,59 @@ impl ParolGrammarTrait for ParolGrammar {
         }
     }
 
+    /// Semantic action for production 6:
+    ///
+    /// Declaration: "%comment" String;
+    ///
+    fn declaration_6(
+        &mut self,
+        _percent_comment_0: &ParseTreeStackEntry,
+        _string_1: &ParseTreeStackEntry,
+        _parse_tree: &Tree<ParseTreeType>,
+    ) -> Result<()> {
+        let context = "declaration_6";
+        if let Some(ParolGrammarItem::Fac(Factor::Terminal(s, _))) = self.pop(context) {
+            self.comment = Some(s);
+            Ok(())
+        } else {
+            Err(format!("{}: Expected 'Fac(Factor::Terminal)' on TOS.", context).into())
+        }
+    }
+
+    /// Semantic action for production 8:
+    ///
+    /// Declaration: "%line_comment" String;
+    ///
+    fn scanner_directives_8(
+        &mut self,
+        _percent_line_underscore_comment_0: &ParseTreeStackEntry,
+        _string_1: &ParseTreeStackEntry,
+        _parse_tree: &Tree<ParseTreeType>,
+    ) -> Result<()> {
+        let context = "scanner_directives_8";
+        if let Some(ParolGrammarItem::Fac(Factor::Terminal(s, _))) = self.pop(context) {
+            self.current_scanner.line_comments.push(s);
+            Ok(())
+        } else {
+            Err(format!("{}: Expected 'Fac(Factor::Terminal)' on TOS.", context).into())
+        }
+    }
+
     /// Semantic action for production 9:
     ///
-    /// Declaration: "%comment" String;
+    /// ScannerDirectives: "%block_comment" String String;
     ///
-    fn declaration_9(
-        &mut self,
-        _percent_comment_0: &ParseTreeStackEntry,
-        _string_1: &ParseTreeStackEntry,
-        _parse_tree: &Tree<ParseTreeType>,
-    ) -> Result<()> {
-        let context = "declaration_9";
-        if let Some(ParolGrammarItem::Fac(Factor::Terminal(s))) = self.ast_stack.pop() {
-            self.comment = Some(s);
-            Ok(())
-        } else {
-            Err(format!("{}: Expected 'Fac(Factor::Terminal)' on TOS.", context).into())
-        }
-    }
-
-    /// Semantic action for production 10:
-    ///
-    /// Declaration: "%line_comment" String;
-    ///
-    fn declaration_10(
-        &mut self,
-        _percent_line_underscore_comment_0: &ParseTreeStackEntry,
-        _string_1: &ParseTreeStackEntry,
-        _parse_tree: &Tree<ParseTreeType>,
-    ) -> Result<()> {
-        let context = "declaration_10";
-        if let Some(ParolGrammarItem::Fac(Factor::Terminal(s))) = self.ast_stack.pop() {
-            self.line_comments.push(s);
-            Ok(())
-        } else {
-            Err(format!("{}: Expected 'Fac(Factor::Terminal)' on TOS.", context).into())
-        }
-    }
-
-    /// Semantic action for production 11:
-    ///
-    /// Declaration: "%block_comment" String String;
-    ///
-    fn declaration_11(
+    fn scanner_directives_9(
         &mut self,
         _percent_block_underscore_comment_0: &ParseTreeStackEntry,
         _string_1: &ParseTreeStackEntry,
         _string_2: &ParseTreeStackEntry,
         _parse_tree: &Tree<ParseTreeType>,
     ) -> Result<()> {
-        let context = "declaration_11";
-        if let Some(ParolGrammarItem::Fac(Factor::Terminal(s1))) = self.ast_stack.pop() {
-            if let Some(ParolGrammarItem::Fac(Factor::Terminal(s2))) = self.ast_stack.pop() {
-                self.block_comments.push((s2, s1));
+        let context = "scanner_directives_9";
+        if let Some(ParolGrammarItem::Fac(Factor::Terminal(s1, _))) = self.pop(context) {
+            if let Some(ParolGrammarItem::Fac(Factor::Terminal(s2, _))) = self.pop(context) {
+                self.current_scanner.block_comments.push((s2, s1));
                 Ok(())
             } else {
                 Err(format!("{}: Expected 'Fac(Factor::Terminal)' on TOS.", context).into())
@@ -291,31 +418,31 @@ impl ParolGrammarTrait for ParolGrammar {
         }
     }
 
-    /// Semantic action for production 12:
+    /// Semantic action for production 10:
     ///
-    /// Declaration: "%auto_newline_off";
+    /// ScannerDirectives: "%auto_newline_off";
     ///
-    fn declaration_12(
+    fn scanner_directives_10(
         &mut self,
         _percent_auto_underscore_newline_underscore_off_0: &ParseTreeStackEntry,
         _parse_tree: &Tree<ParseTreeType>,
     ) -> Result<()> {
-        let _context = "declaration_12";
-        self.auto_newline_off = true;
+        let _context = "scanner_directives_10";
+        self.current_scanner.auto_newline_off = true;
         Ok(())
     }
 
-    /// Semantic action for production 13:
+    /// Semantic action for production 11:
     ///
-    /// Declaration: "%auto_ws_off";
+    /// ScannerDirectives: "%auto_ws_off";
     ///
-    fn declaration_13(
+    fn scanner_directives_11(
         &mut self,
         _percent_auto_underscore_ws_underscore_off_0: &ParseTreeStackEntry,
         _parse_tree: &Tree<ParseTreeType>,
     ) -> Result<()> {
-        let _context = "declaration_13";
-        self.auto_ws_off = true;
+        let _context = "scanner_directives_11";
+        self.current_scanner.auto_ws_off = true;
         Ok(())
     }
 
@@ -332,10 +459,10 @@ impl ParolGrammarTrait for ParolGrammar {
         _parse_tree: &Tree<ParseTreeType>,
     ) -> Result<()> {
         let context = "production_20";
-        if let Some(ParolGrammarItem::Alts(mut rhs)) = self.ast_stack.pop() {
-            if let Some(ParolGrammarItem::Fac(Factor::NonTerminal(lhs))) = self.ast_stack.pop() {
+        if let Some(ParolGrammarItem::Alts(mut rhs)) = self.pop(context) {
+            if let Some(ParolGrammarItem::Fac(Factor::NonTerminal(lhs))) = self.pop(context) {
                 rhs.reverse();
-                self.ast_stack
+                self.item_stack
                     .push(ParolGrammarItem::Prod(Production::new(lhs, rhs)));
                 Ok(())
             } else {
@@ -357,11 +484,11 @@ impl ParolGrammarTrait for ParolGrammar {
         _parse_tree: &Tree<ParseTreeType>,
     ) -> Result<()> {
         let context = "alternations_21";
-        if let Some(ParolGrammarItem::Alts(mut alts)) = self.ast_stack.pop() {
-            if let Some(ParolGrammarItem::Alt(mut alt)) = self.ast_stack.pop() {
+        if let Some(ParolGrammarItem::Alts(mut alts)) = self.pop(context) {
+            if let Some(ParolGrammarItem::Alt(mut alt)) = self.pop(context) {
                 alt.reverse();
                 alts.push(alt);
-                self.ast_stack.push(ParolGrammarItem::Alts(alts));
+                self.push(ParolGrammarItem::Alts(alts), context);
                 Ok(())
             } else {
                 Err(format!("{}: Expected 'Alt' on TOS.", context).into())
@@ -377,7 +504,7 @@ impl ParolGrammarTrait for ParolGrammar {
     ///
     fn alternations_suffix_23(&mut self, _parse_tree: &Tree<ParseTreeType>) -> Result<()> {
         let _context = "alternations_suffix_23";
-        self.ast_stack
+        self.item_stack
             .push(ParolGrammarItem::Alts(Alternations::new()));
         Ok(())
     }
@@ -388,17 +515,17 @@ impl ParolGrammarTrait for ParolGrammar {
     ///
     fn alternations_rest_24(
         &mut self,
-        _esc_or_0: &ParseTreeStackEntry,
+        _or_0: &ParseTreeStackEntry,
         _alternation_1: &ParseTreeStackEntry,
         _alternations_rest_suffix_2: &ParseTreeStackEntry,
         _parse_tree: &Tree<ParseTreeType>,
     ) -> Result<()> {
         let context = "alternations_rest_24";
-        if let Some(ParolGrammarItem::Alts(mut alts)) = self.ast_stack.pop() {
-            if let Some(ParolGrammarItem::Alt(mut alt)) = self.ast_stack.pop() {
+        if let Some(ParolGrammarItem::Alts(mut alts)) = self.pop(context) {
+            if let Some(ParolGrammarItem::Alt(mut alt)) = self.pop(context) {
                 alt.reverse();
                 alts.push(alt);
-                self.ast_stack.push(ParolGrammarItem::Alts(alts));
+                self.push(ParolGrammarItem::Alts(alts), context);
                 Ok(())
             } else {
                 Err(format!("{}: Expected 'Alt' on TOS.", context).into())
@@ -414,7 +541,7 @@ impl ParolGrammarTrait for ParolGrammar {
     ///
     fn alternations_rest_suffix_26(&mut self, _parse_tree: &Tree<ParseTreeType>) -> Result<()> {
         let _context = "alternations_rest_suffix_26";
-        self.ast_stack
+        self.item_stack
             .push(ParolGrammarItem::Alts(Alternations::new()));
         Ok(())
     }
@@ -424,8 +551,8 @@ impl ParolGrammarTrait for ParolGrammar {
     /// Alternation: ;
     ///
     fn alternation_28(&mut self, _parse_tree: &Tree<ParseTreeType>) -> Result<()> {
-        let _context = "alternation_27";
-        self.ast_stack
+        let _context = "alternation_28";
+        self.item_stack
             .push(ParolGrammarItem::Alt(Alternation::new()));
         Ok(())
     }
@@ -441,11 +568,11 @@ impl ParolGrammarTrait for ParolGrammar {
         _parse_tree: &Tree<ParseTreeType>,
     ) -> Result<()> {
         let context = "alternation_rest_29";
-        trace!("{}", self.trace_ast_stack(context));
-        if let Some(ParolGrammarItem::Alt(mut alt)) = self.ast_stack.pop() {
-            if let Some(ParolGrammarItem::Fac(fac)) = self.ast_stack.pop() {
+        //trace!("{}", self.trace_item_stack(context));
+        if let Some(ParolGrammarItem::Alt(mut alt)) = self.pop(context) {
+            if let Some(ParolGrammarItem::Fac(fac)) = self.pop(context) {
                 alt.push(fac);
-                self.ast_stack.push(ParolGrammarItem::Alt(alt));
+                self.push(ParolGrammarItem::Alt(alt), context);
                 Ok(())
             } else {
                 Err(format!("{}: Expected 'Fac' on TOS.", context).into())
@@ -461,25 +588,52 @@ impl ParolGrammarTrait for ParolGrammar {
     ///
     fn alternation_rest_suffix_31(&mut self, _parse_tree: &Tree<ParseTreeType>) -> Result<()> {
         let _context = "alternation_rest_suffix_31";
-        self.ast_stack
+        self.item_stack
             .push(ParolGrammarItem::Alt(Alternation::new()));
         Ok(())
     }
 
-    /// Semantic action for production 38:
+    /// Semantic action for production 41:
+    ///
+    /// TokenWithStates: "<" Identifier ">" String;
+    ///
+    fn token_with_states_41(
+        &mut self,
+        _l_t_0: &ParseTreeStackEntry,
+        _state_list_1: &ParseTreeStackEntry,
+        _g_t_2: &ParseTreeStackEntry,
+        _string_3: &ParseTreeStackEntry,
+        _parse_tree: &Tree<ParseTreeType>,
+    ) -> Result<()> {
+        let context = "token_with_state_41";
+        trace!("{}", self.trace_item_stack(context));
+        if let Some(ParolGrammarItem::Fac(Factor::Terminal(s, _))) = self.pop(context) {
+            if let Some(ParolGrammarItem::StateList(sc)) = self.pop(context) {
+                self.item_stack
+                    .push(ParolGrammarItem::Fac(Factor::Terminal(s, sc)));
+                Ok(())
+            } else {
+                Err(format!("{}: Expected 'StateList' on TOS.", context).into())
+            }
+        } else {
+            Err(format!("{}: Expected 'Factor::Terminal' on TOS.", context).into())
+        }
+    }
+
+    /// Semantic action for production 42:
     ///
     /// Group: "\(" Alternations "\)";
     ///
-    fn group_38(
+    fn group_42(
         &mut self,
-        _esc_l_paren_0: &ParseTreeStackEntry,
+        _l_paren_0: &ParseTreeStackEntry,
         _alternations_1: &ParseTreeStackEntry,
-        _esc_r_paren_2: &ParseTreeStackEntry,
+        _r_paren_2: &ParseTreeStackEntry,
         _parse_tree: &Tree<ParseTreeType>,
     ) -> Result<()> {
-        let context = "group_38";
-        if let Some(ParolGrammarItem::Alts(alts)) = self.ast_stack.pop() {
-            self.ast_stack
+        let context = "group_42";
+        if let Some(ParolGrammarItem::Alts(alts)) = self.pop(context) {
+            self.item_stack
                 .push(ParolGrammarItem::Fac(Factor::Group(alts)));
             Ok(())
         } else {
@@ -487,20 +641,20 @@ impl ParolGrammarTrait for ParolGrammar {
         }
     }
 
-    /// Semantic action for production 39:
+    /// Semantic action for production 43:
     ///
     /// Optional: "\[" Alternations "\]";
     ///
-    fn optional_39(
+    fn optional_43(
         &mut self,
-        _esc_l_bracket_0: &ParseTreeStackEntry,
+        _l_bracket_0: &ParseTreeStackEntry,
         _alternations_1: &ParseTreeStackEntry,
-        _esc_r_bracket_2: &ParseTreeStackEntry,
+        _r_bracket_2: &ParseTreeStackEntry,
         _parse_tree: &Tree<ParseTreeType>,
     ) -> Result<()> {
-        let context = "optional_39";
-        if let Some(ParolGrammarItem::Alts(alts)) = self.ast_stack.pop() {
-            self.ast_stack
+        let context = "optional_43";
+        if let Some(ParolGrammarItem::Alts(alts)) = self.pop(context) {
+            self.item_stack
                 .push(ParolGrammarItem::Fac(Factor::Optional(alts)));
             Ok(())
         } else {
@@ -508,20 +662,20 @@ impl ParolGrammarTrait for ParolGrammar {
         }
     }
 
-    /// Semantic action for production 40:
+    /// Semantic action for production 44:
     ///
     /// Repeat: "\{" Alternations "\}";
     ///
-    fn repeat_40(
+    fn repeat_44(
         &mut self,
-        _esc_l_brace_0: &ParseTreeStackEntry,
+        _l_brace_0: &ParseTreeStackEntry,
         _alternations_1: &ParseTreeStackEntry,
-        _esc_r_brace_2: &ParseTreeStackEntry,
+        _r_brace_2: &ParseTreeStackEntry,
         _parse_tree: &Tree<ParseTreeType>,
     ) -> Result<()> {
-        let context = "repeat_40";
-        if let Some(ParolGrammarItem::Alts(alts)) = self.ast_stack.pop() {
-            self.ast_stack
+        let context = "repeat_44";
+        if let Some(ParolGrammarItem::Alts(alts)) = self.pop(context) {
+            self.item_stack
                 .push(ParolGrammarItem::Fac(Factor::Repeat(alts)));
             Ok(())
         } else {
@@ -529,45 +683,155 @@ impl ParolGrammarTrait for ParolGrammar {
         }
     }
 
-    /// Semantic action for production 41:
+    /// Semantic action for production 45:
     ///
     /// Identifier: "[a-zA-Z_]\w*";
     ///
-    fn identifier_41(
+    fn identifier_45(
         &mut self,
         identifier_0: &ParseTreeStackEntry,
         parse_tree: &Tree<ParseTreeType>,
     ) -> Result<()> {
-        let context = "identifier_41";
-        let ast_item = identifier_0.get_ast_type(parse_tree);
-        if let ParseTreeType::T(t) = ast_item {
-            self.ast_stack
+        let context = "identifier_45";
+        let parse_tree_item = identifier_0.get_parse_tree_type(parse_tree);
+        if let ParseTreeType::T(t) = parse_tree_item {
+            self.item_stack
                 .push(ParolGrammarItem::Fac(Factor::NonTerminal(t.symbol.clone())));
             Ok(())
         } else {
-            Err(format!("{}: Token expected, found {}", context, ast_item).into())
+            Err(format!("{}: Token expected, found {}", context, parse_tree_item).into())
         }
     }
 
-    /// Semantic action for production 42:
+    /// Semantic action for production 46:
     ///
     /// String: "\u{0022}([^\\]|\\.)*?\u{0022}";
     ///
-    fn string_42(
+    fn string_46(
         &mut self,
         string_0: &ParseTreeStackEntry,
         parse_tree: &Tree<ParseTreeType>,
     ) -> Result<()> {
-        let context = "string_42";
-        let ast_item = string_0.get_ast_type(parse_tree);
-        if let ParseTreeType::T(t) = ast_item {
+        let context = "string_46";
+        let parse_tree_item = string_0.get_parse_tree_type(parse_tree);
+        if let ParseTreeType::T(t) = parse_tree_item {
             // Trim double quotes here
             let s = t.symbol.clone().trim_matches('"').to_owned();
-            self.ast_stack
-                .push(ParolGrammarItem::Fac(Factor::Terminal(s)));
+            self.item_stack
+                .push(ParolGrammarItem::Fac(Factor::Terminal(s, vec![0])));
             Ok(())
         } else {
-            Err(format!("{}: Token expected, found {}", context, ast_item).into())
+            Err(format!("{}: Token expected, found {}", context, parse_tree_item).into())
         }
     }
+
+    /// Semantic action for production 48:
+    ///
+    /// ScannerStateSuffix: ScannerStateRest "\}";
+    ///
+    fn scanner_state_suffix_48(
+        &mut self,
+        _scanner_state_rest_0: &ParseTreeStackEntry,
+        _r_brace_1: &ParseTreeStackEntry,
+        _parse_tree: &Tree<ParseTreeType>,
+    ) -> Result<()> {
+        let context = "scanner_state_suffix_48";
+        trace!("{}", self.trace_item_stack(context));
+        if let Some(ParolGrammarItem::Fac(Factor::NonTerminal(n))) = self.pop(context) {
+            trace!("{}", self);
+            self.current_scanner.name = n;
+            self.scanner_configurations
+                .push(self.current_scanner.clone());
+            self.current_scanner = ScannerConfig::default();
+            trace!("{}", self);
+            Ok(())
+        } else {
+            Err(format!("{}: Expected 'Factor::NonTerminal' on TOS.", context).into())
+        }
+    }
+
+    /// Semantic action for production 53:
+    ///
+    /// StateList: Identifier StateListRest;
+    ///
+    fn state_list_53(
+        &mut self,
+        _identifier_0: &ParseTreeStackEntry,
+        _state_list_rest_1: &ParseTreeStackEntry,
+        _parse_tree: &Tree<ParseTreeType>,
+    ) -> Result<()> {
+        let context = "state_list_53";
+        trace!("{}", self.trace_item_stack(context));
+        self.handle_scanner_state(context)
+    }
+
+    /// Semantic action for production 54:
+    ///
+    /// StateListRest: "," Identifier StateListRest;
+    ///
+    fn state_list_rest_54(
+        &mut self,
+        _comma_0: &ParseTreeStackEntry,
+        _identifier_1: &ParseTreeStackEntry,
+        _state_list_rest_2: &ParseTreeStackEntry,
+        _parse_tree: &Tree<ParseTreeType>,
+    ) -> Result<()> {
+        let context = "state_list_rest_54";
+        trace!("{}", self.trace_item_stack(context));
+        self.handle_scanner_state(context)
+    }
+
+    /// Semantic action for production 55:
+    ///
+    /// StateListRest: ;
+    ///
+    fn state_list_rest_55(&mut self, _parse_tree: &Tree<ParseTreeType>) -> Result<()> {
+        let context = "state_list_rest_55";
+        // Start with an empty state list
+        self.push(ParolGrammarItem::StateList(vec![]), context);
+        Ok(())
+    }
+
+    /// Semantic action for production 56:
+    ///
+    /// ScannerSwitch: "%sc" "\(" ScannerNameOpt "\)";
+    ///
+    fn scanner_switch_56(
+        &mut self,
+        _percent_sc_0: &ParseTreeStackEntry,
+        _l_paren_1: &ParseTreeStackEntry,
+        _scanner_name_opt_2: &ParseTreeStackEntry,
+        _r_paren_3: &ParseTreeStackEntry,
+        _parse_tree: &Tree<ParseTreeType>,
+    ) -> Result<()> {
+        let context = "scanner_switch_56";
+        if let Some(ParolGrammarItem::Fac(Factor::NonTerminal(s))) = self.pop(context) {
+            if let Some(scanner_state) = self
+                .scanner_configurations
+                .iter()
+                .position(|sc| sc.name == *s)
+            {
+                self.item_stack
+                    .push(ParolGrammarItem::Fac(Factor::ScannerSwitch(scanner_state)));
+                trace!("{}", self.trace_item_stack(context));
+                Ok(())
+            } else {
+                Err(format!("{}: Unknown scanner name '{}'", context, s).into())
+            }
+        } else {
+            Err(format!("{}: Expected 'Fac(Factor::NonTerminal)' on TOS.", context).into())
+        }
+    }
+
+    /// Semantic action for production 58:
+    ///
+    /// ScannerNameOpt: ;
+    ///
+    fn scanner_name_opt_58(&mut self, _parse_tree: &Tree<ParseTreeType>) -> Result<()> {
+        self.item_stack
+            .push(ParolGrammarItem::Fac(Factor::NonTerminal(
+                "INITIAL".to_string(),
+            )));
+        Ok(())
+    }
 }
